{% extends "base.html" %}
{% load static %}
{% load crispy_forms_tags %}
{% block body %}

<div class="card bg-light">
  <div class="card-header">Create Group</div>
  <div class="card-body">
    <p class="card-text">
      {% crispy form %}
    </p>
  </div>
</div>
{% endblock %}
